@@ -97,12 +97,7 @@ void on_addbutton_clicked(GtkButton *button, gpointer user_data)
 	GladeXML *gxml = glade_xml_new (GLADE_FILE_RULE_ADD, NULL, NULL);
 
 	GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
+	
 	gtk_widget_show(window);
 
 	xyz_t *ud = malloc(sizeof(xyz_t));
@@ -199,12 +194,7 @@ void on_editbutton_clicked(GtkButton *button, gpointer user_data)
 		GladeXML *gxml = glade_xml_new (GLADE_FILE_RULE_ADD, NULL, NULL);
 
 		GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
-		GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-		if (window_icon_pixbuf)
-		{
-			gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-			gdk_pixbuf_unref (window_icon_pixbuf);
-		}
+
 		GtkWidget *entry = glade_xml_get_widget (gxml, "entry1");
 		gtk_entry_set_text(GTK_ENTRY(entry), letters);
 
@@ -261,6 +251,23 @@ void on_deletebutton_clicked(GtkButton *button, gpointer user_data)
 		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 }
 
+void on_extension_install_check(gpointer user_data)
+{
+	GtkWidget *checkbutton = (GtkWidget *)user_data;
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (checkbutton)))
+	{
+		char *buffer = _("To enable the extension you need to start the session again.");
+
+		GtkWidget *dialog = gtk_message_dialog_new (NULL,
+											GTK_DIALOG_DESTROY_WITH_PARENT,
+											GTK_MESSAGE_INFO,
+											GTK_BUTTONS_OK,
+											"%s", buffer);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+	}
+}
+
 // Save dictionary/regexp list
 static gboolean save_regexp(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
 {
