@@ -21,11 +21,12 @@
 #   include "config.h"
 #endif
 
+#include <glib/gstdio.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <gconf/gconf-client.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -46,6 +47,10 @@
 
 #define GXNEUR_DESKTOP "[Desktop Entry]\nType=Application\nExec="  PACKAGE "\nHidden=false\nX-GNOME-Autostart-enabled=true\nName=GTK UI for X Neural Switcher\n"
 
+#define GNOME3_EXT_PATH ".local/share/gnome-shell/extensions/" PACKAGE "@xneur.ru"  
+#define GNOME3_EXT_JS_FILE "extension.js"
+#define GNOME3_EXT_JSON_FILE "metadata.json"
+
 #define DEFAULT_MAX_PATH	4096
 
 #include "support.h"
@@ -54,12 +59,20 @@
 
 #include "misc.h"
 
+#include "configuration.h"
+
+extern int arg_delay;
+extern const char* arg_keyboard_properties;
+extern const char* arg_show_in_the_tray;
+extern const char* arg_rendering_engine;
+
+
 void xneur_edit_regexp(GtkWidget *treeview);
 void xneur_edit_rule(GtkWidget *treeview);
 void xneur_edit_dictionary(GtkWidget *treeview);
 
 #define MAX_LANGUAGES			4
-#define XNEUR_NEEDED_MAJOR_VERSION	12
+#define XNEUR_NEEDED_MAJOR_VERSION	15
 #define XNEUR_BUILD_MINOR_VERSION	0
 	
 struct _xneur_config *xconfig				= NULL;
@@ -75,7 +88,7 @@ static GtkListStore *store_osd				= NULL;
 static GtkListStore *store_popup			= NULL;
 static GtkListStore *store_action			= NULL;
 static GtkListStore *store_hotkey			= NULL;
-static GtkListStore *store_autocomplementation_exclude_app		= NULL;
+static GtkListStore *store_autocompletion_exclude_app		= NULL;
 static GtkListStore *store_plugin			= NULL;
 static GtkListStore *store_language			= NULL;
 
@@ -96,8 +109,10 @@ static const char *notify_names[]			=   {
 										"Correct selected text", "Transliterate selected text", "Change case of selected text", "Preview correction of selected text",
 										"Correct clipboard text", "Transliterate clipboard text", "Change case of clipboard text", "Preview correction of clipboard text",
 	                                    "Expand abbreviations",
-										"Correct aCCIDENTAL caps", "Correct TWo INitial caps", "Execute user action", 
-										"Block keyboard and mouse events", "Unblock keyboard and mouse events"
+										"Correct aCCIDENTAL caps", "Correct TWo INitial caps", "Сorrect two space with a comma and a space",
+										"Сorrect two minus with a dash", "Сorrect (c) with a copyright sign", "Сorrect (tm) with a trademark sign",
+										"Сorrect (r) with a registered sign", 
+										"Execute user action", "Block keyboard and mouse events", "Unblock keyboard and mouse events"
 										};
 
 static const char *hotkey_names[]			=   {
@@ -106,7 +121,7 @@ static const char *hotkey_names[]			=   {
 										"Correct selected text", "Transliterate selected text", "Change case of selected text", "Preview correction of selected text",
 	                                    "Correct clipboard text", "Transliterate clipboard text", "Change case of clipboard text", "Preview correction of clipboard text",
 										"Switch to layout 1", "Switch to layout 2", "Switch to layout 3", "Switch to layout 4",
-		                                "Rotate layouts", "Rotate layouts back", "Expand abbreviations", "Autocomplementation confirmation", 
+		                                "Rotate layouts", "Rotate layouts back", "Expand abbreviations", "Autocompletion confirmation", 
 										"Block/Unblock keyboard and mouse events", "Insert date"
                                         };
 
@@ -221,10 +236,8 @@ static void get_xprop_name(GladeXML *gxml)
 	gtk_entry_set_text(GTK_ENTRY(entry1), p);
 }
 
-static void get_logfile(GladeXML *gxml)
+void xneur_get_logfile()
 {
-	if (gxml) {};
-	
 	char *log_home_path	= xconfig->get_home_dict_path(NULL, "xneurlog.html");
 	char *command = malloc ((strlen("xdg-open ") + strlen(log_home_path) + strlen(" 2> /dev/stdout") + 1) * sizeof(char));
 	command[0] = '\0';
@@ -272,13 +285,6 @@ static void xneur_add_application(GtkListStore *store)
 	glade_xml_signal_autoconnect (gxml);
 	GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
 
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
-	
 	gtk_widget_show(window);
 
 	GtkWidget *widget = glade_xml_get_widget (gxml, "button3");
@@ -363,17 +369,6 @@ void xneur_about(void)
 	glade_xml_signal_autoconnect (gxml);
 	GtkWidget *window = glade_xml_get_widget (gxml, "window1");
 
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		
-		GtkWidget *image = glade_xml_get_widget (gxml, "image1");	
-		gtk_image_set_from_pixbuf (GTK_IMAGE (image), window_icon_pixbuf);
-		
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
-	
 	GtkWidget *widget = glade_xml_get_widget(gxml, "label44");
 	gchar *text = g_strdup_printf("%s %s", _("Current Version"), VERSION);
 	gtk_label_set_text(GTK_LABEL(widget), text);	
@@ -437,52 +432,11 @@ static void notify_enable (GtkCellRendererToggle *renderer, gchar *path, GtkTree
 		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 2, value, -1);
 }
 
-static void xneur_replace_pixmap_dir(GladeXML *gxml)
-{
-	GtkWidget *window = glade_xml_get_widget (gxml, "filechooserdialog1");
-
-	gtk_entry_set_text(GTK_ENTRY(tmp_widget), gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (window)));
-	
-	gtk_widget_destroy(window);
-}
-
-static void xneur_restore_pixmap_dir(GladeXML *gxml)
-{
-	GtkWidget *widget = glade_xml_get_widget (gxml, "entry1");
-
-	gtk_entry_set_text(GTK_ENTRY(widget), PACKAGE_PIXMAPS_DIR);
-}
-
-static void xneur_edit_pixmap_dir(GladeXML *parent_gxml)
-{
-	GladeXML *gxml = glade_xml_new (GLADE_FILE_CHOOSE, NULL, NULL);
-	
-	glade_xml_signal_autoconnect (gxml);
-	GtkWidget *window = glade_xml_get_widget (gxml, "filechooserdialog1");
-
-	tmp_widget = glade_xml_get_widget (parent_gxml, "entry1");
-	
-	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(window), gtk_entry_get_text(GTK_ENTRY(tmp_widget)));
-	gtk_file_chooser_set_action(GTK_FILE_CHOOSER(window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);	
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
-	
-	gtk_widget_show(window);
-		
-	// Button OK
-	GtkWidget *widget = glade_xml_get_widget (gxml, "button5");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_replace_pixmap_dir), gxml);
-}
-
 static void xneur_restore_keyboard_properties(GladeXML *gxml)
 {
 	GtkWidget *widget = glade_xml_get_widget (gxml, "entry5");
 
-	gtk_entry_set_text(GTK_ENTRY(widget), KB_PROP_COMMAND);
+	gtk_entry_set_text(GTK_ENTRY(widget), DEFAULT_KEYBOARD_PROPERTIES);
 }
 
 static void xneur_replace_keyboard_properties(GladeXML *gxml)
@@ -503,13 +457,7 @@ static void xneur_edit_keyboard_properties(GladeXML *parent_gxml)
 
 	tmp_widget = glade_xml_get_widget (parent_gxml, "entry5");
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window), gtk_entry_get_text(GTK_ENTRY(tmp_widget)));
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
-	
+		
 	gtk_widget_show(window);
 		
 	// Button OK
@@ -519,25 +467,22 @@ static void xneur_edit_keyboard_properties(GladeXML *parent_gxml)
 
 void xneur_kb_preference(void)
 {
-	GConfClient* gconfClient = gconf_client_get_default();
-	g_assert(GCONF_IS_CLIENT(gconfClient));
-
-	GConfValue* gcValue = NULL;
-	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "keyboard_properties", NULL);
-	if(gcValue != NULL) 
-	{
-		const char *string_value = NULL;
-		if(gcValue->type == GCONF_VALUE_STRING) 
-		{
-			string_value = gconf_value_get_string(gcValue);
-			if (!g_spawn_command_line_async(string_value, NULL))
-			{
-				error_msg(_("Couldn't start %s\nVerify that it installed\n"), string_value);
-				fprintf(stderr, _("Couldn't start %s\nVerify that it installed\n"), string_value);
-			}
-		}
-		gconf_value_free(gcValue);
+	gchar *string_value = NULL;
+	gxneur_config_read_str("keyboard_properties", &string_value);
+	if (arg_keyboard_properties) {
+		g_free(string_value);
+		string_value = g_strdup(arg_keyboard_properties);
 	}
+
+	gchar *cmd = string_value ? string_value : DEFAULT_KEYBOARD_PROPERTIES;
+
+	if (!g_spawn_command_line_async(cmd, NULL))
+	{
+		error_msg(_("Couldn't start %s\nVerify that it installed\n"), cmd);
+		fprintf(stderr, _("Couldn't start %s\nVerify that it installed\n"), cmd);
+	}
+
+	g_free(string_value);
 }
 
 void xneur_preference(void)
@@ -547,13 +492,6 @@ void xneur_preference(void)
 	glade_xml_signal_autoconnect (gxml);
 	GtkWidget *window = glade_xml_get_widget (gxml, "window2");
 
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
-	
 	gtk_widget_show(window);
 
 	// Mode set
@@ -756,7 +694,7 @@ void xneur_preference(void)
 
 	// View log
 	widget = glade_xml_get_widget (gxml, "button8");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(get_logfile), gxml);
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_get_logfile), NULL);
 	
 	// Ignore Keyboard Layout Mode
 	widget = glade_xml_get_widget (gxml, "checkbutton8");
@@ -778,26 +716,41 @@ void xneur_preference(void)
 	widget = glade_xml_get_widget (gxml, "checkbutton31");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_capital_letter_after_dot);
 
-	// Autocomplementation
+	widget = glade_xml_get_widget (gxml, "checkbutton29");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_two_space_with_comma_and_space);
+
+	widget = glade_xml_get_widget (gxml, "checkbutton37");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_two_minus_with_dash);
+
+	widget = glade_xml_get_widget (gxml, "checkbutton38");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_c_with_copyright);
+
+	widget = glade_xml_get_widget (gxml, "checkbutton39");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_tm_with_trademark);
+
+	widget = glade_xml_get_widget (gxml, "checkbutton40");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_r_with_registered);
+	
+	// Autocompletion
 	widget = glade_xml_get_widget (gxml, "checkbutton21");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->autocomplementation);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->autocompletion);
 
-	// Space after autocomplementation
+	// Space after autocompletion
 	widget = glade_xml_get_widget (gxml, "checkbutton1");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->add_space_after_autocomplementation);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->add_space_after_autocompletion);
 
-	// Exclude autocomplementation App set
+	// Exclude autocompletion App set
 	treeview = glade_xml_get_widget (gxml, "treeview8");
 
-	store_autocomplementation_exclude_app = gtk_list_store_new(1, G_TYPE_STRING);
-	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(store_autocomplementation_exclude_app));
+	store_autocompletion_exclude_app = gtk_list_store_new(1, G_TYPE_STRING);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(store_autocompletion_exclude_app));
 	gtk_widget_show(treeview);
 
-	for (int i = 0; i < xconfig->autocomplementation_excluded_apps->data_count; i++)
+	for (int i = 0; i < xconfig->autocompletion_excluded_apps->data_count; i++)
 	{
 		GtkTreeIter iter;
-		gtk_list_store_append(GTK_LIST_STORE(store_autocomplementation_exclude_app), &iter);
-		gtk_list_store_set(GTK_LIST_STORE(store_autocomplementation_exclude_app), &iter, 0, xconfig->autocomplementation_excluded_apps->data[i].string, -1);
+		gtk_list_store_append(GTK_LIST_STORE(store_autocompletion_exclude_app), &iter);
+		gtk_list_store_set(GTK_LIST_STORE(store_autocompletion_exclude_app), &iter, 0, xconfig->autocompletion_excluded_apps->data[i].string, -1);
 	}
 
 	cell = gtk_cell_renderer_text_new();
@@ -807,10 +760,10 @@ void xneur_preference(void)
 
 	// Adding/Removing Exclude App
 	widget = glade_xml_get_widget (gxml, "button11");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_add_autocomplementation_exclude_app), G_OBJECT(window));
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_add_autocompletion_exclude_app), G_OBJECT(window));
 
 	widget = glade_xml_get_widget (gxml, "button13");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_rem_autocomplementation_exclude_app), G_OBJECT(treeview));
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_rem_autocompletion_exclude_app), G_OBJECT(treeview));
 
 	// Hotkeys List set
 	treeview = glade_xml_get_widget (gxml, "treeview5");
@@ -1115,6 +1068,10 @@ void xneur_preference(void)
 	// Show popup
 	widget = glade_xml_get_widget (gxml, "checkbutton22");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->show_popup);
+
+	// Popup expiration delay
+	widget = glade_xml_get_widget (gxml, "spinbutton6");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), xconfig->popup_expire_timeout);
 	
 	// Popup Text Preference
 	// Popup List set
@@ -1173,10 +1130,20 @@ void xneur_preference(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->troubleshoot_delete);
 	widget = glade_xml_get_widget (gxml, "checkbutton25");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->troubleshoot_switch);
+	widget = glade_xml_get_widget (gxml, "checkbutton35");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->troubleshoot_full_screen);
 
 	// Dont Send KeyRelease Mode
 	widget = glade_xml_get_widget (gxml, "checkbutton26");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->dont_send_key_release);
+
+	// Tracking input mode set
+	widget = glade_xml_get_widget (gxml, "checkbutton34");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->tracking_input);
+
+	// Tracking mouse mode set
+	widget = glade_xml_get_widget (gxml, "checkbutton28");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->tracking_mouse);
 	
 	// Plugins
 	treeview = glade_xml_get_widget (gxml, "treeview11");
@@ -1273,105 +1240,97 @@ void xneur_preference(void)
 		widget = glade_xml_get_widget (gxml, "checkbutton27");
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 	}
+
+	// Gnome 3 shell
+	widget = glade_xml_get_widget (gxml, "checkbutton36");
+	path_file = g_build_filename(getenv("HOME"), GNOME3_EXT_PATH, NULL);
+	if (g_file_test(path_file, G_FILE_TEST_IS_DIR))
+	{
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
+	}
+	g_free(path_file);
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(on_extension_install_check), G_OBJECT(widget));
 	
 	// Delay before start
-	GConfClient* gconfClient = gconf_client_get_default();
-	g_assert(GCONF_IS_CLIENT(gconfClient));
-
-	// Delay before start
-	GConfValue* gcValue = NULL;
-	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "delay", NULL);
-
-	int value = 0;
-	if(gcValue != NULL) 
-	{
-		/* Check if value type is integer */
-		if(gcValue->type == GCONF_VALUE_INT) 
-			value = gconf_value_get_int(gcValue);
-
-		/* Release resources */
-		gconf_value_free(gcValue);
-	}
-
 	widget = glade_xml_get_widget (gxml, "spinbutton5");
+	int value = DEFAULT_DELAY;
+	if (gxneur_config_read_int("delay", &value) == CONFIG_NOT_SUPPORTED)
+		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE);
+	if (arg_delay >= 0)
+		value = arg_delay;
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), value);
 
-	// Text on tray
-	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "text_on_tray", NULL);
-
-	value = FALSE;
-	if(gcValue != NULL) 
-	{
-		if(gcValue->type == GCONF_VALUE_BOOL) 
-			value = gconf_value_get_bool(gcValue);
-
-		gconf_value_free(gcValue);
-	}		
-
-	widget = glade_xml_get_widget (gxml, "checkbutton28");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), value);
-
-	// Resize icon on tray
-	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "resize_tray_icon", NULL);
-
-	value = FALSE;
-	if(gcValue != NULL) 
-	{
-		if(gcValue->type == GCONF_VALUE_BOOL) 
-			value = gconf_value_get_bool(gcValue);
-
-		gconf_value_free(gcValue);
-	}		
-
-	widget = glade_xml_get_widget (gxml, "checkbutton29");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), value);
-
-	//
-	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "pixmap_dir", NULL);
-
-	/* if value pointer remains NULL, the key was not found */
-	const char *string_value = NULL;
-	if(gcValue != NULL) 
-	{
-		if(gcValue->type == GCONF_VALUE_STRING) 
-		{
-			string_value = gconf_value_get_string(gcValue);
-			widget = glade_xml_get_widget (gxml, "entry1");
-			gtk_entry_set_text(GTK_ENTRY(widget), string_value);
-		}
+	// Show on tray
+	widget = glade_xml_get_widget (gxml, "combobox2");
+	int show_in_the_tray = 0;
+	gchar *string_value = NULL;
+	if (gxneur_config_read_str("show_in_the_tray", &string_value) == CONFIG_NOT_SUPPORTED)
+		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE);
+	if (arg_show_in_the_tray) {
+		g_free(string_value);
+		string_value = g_strdup(arg_show_in_the_tray);
 	}
-	
-	// Button Pixmap Directory Edit
-	widget = glade_xml_get_widget (gxml, "button9");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_pixmap_dir), gxml);
-	// Button Pixmap Directory Edit
-	widget = glade_xml_get_widget (gxml, "button14");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_restore_pixmap_dir), gxml);
+	if (!string_value)
+		string_value = g_strdup(DEFAULT_SHOW_IN_THE_TRAY);
+	if (strcasecmp(string_value, "Flag") == 0)
+		show_in_the_tray = 0;
+	else if (strcasecmp(string_value, "Text") == 0)
+		show_in_the_tray = 1;
+	else
+		show_in_the_tray = 2;
+	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), show_in_the_tray);
+	g_free(string_value);
 
+
+	// Define rendering engine
+	widget = glade_xml_get_widget (gxml, "combobox3");
+	int rendering_engine = 0;
+	string_value = NULL;
+	if (gxneur_config_read_str("rendering_engine", &string_value) == CONFIG_NOT_SUPPORTED)
+		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE);
+	if (arg_rendering_engine) {
+		g_free(string_value);
+		string_value = g_strdup(arg_rendering_engine);
+	}
+	if (!string_value)
+		string_value = g_strdup(DEFAULT_RENDERING_ENGINE);
+	if (strcasecmp(string_value, "Built-in") == 0)
+		rendering_engine = 0;
+	else if (strcasecmp(string_value, "StatusIcon") == 0)
+		rendering_engine = 1;
+	else
+		rendering_engine = 2;
+	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), rendering_engine);
 	
 	// Keyboard properties
-	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "keyboard_properties", NULL);
-
-	if(gcValue != NULL) 
-	{
-		if(gcValue->type == GCONF_VALUE_STRING) 
-		{
-			string_value = gconf_value_get_string(gcValue);
-			widget = glade_xml_get_widget (gxml, "entry5");
-			gtk_entry_set_text(GTK_ENTRY(widget), string_value);
-		}
+	widget = glade_xml_get_widget (gxml, "entry5");
+	string_value = NULL;
+	int keyboard_properties_config_not_supported = 0;
+	if (gxneur_config_read_str("keyboard_properties", &string_value) == CONFIG_NOT_SUPPORTED)
+		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE),
+		keyboard_properties_config_not_supported = 1;
+	if (arg_keyboard_properties) {
+		g_free(string_value);
+		string_value = g_strdup(arg_keyboard_properties);
 	}
-	
-	// Button Pixmap Directory Edit
+	gtk_entry_set_text(GTK_ENTRY(widget), string_value ? string_value : DEFAULT_KEYBOARD_PROPERTIES);
+	g_free(string_value);
+
+
+	// Button Keyboard properties Edit
 	widget = glade_xml_get_widget (gxml, "button15");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_keyboard_properties), gxml);
-	// Button Pixmap Directory Edit
+	if (!keyboard_properties_config_not_supported)
+		g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_keyboard_properties), gxml);
+	else
+		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE);
+
+	// Button Keyboard properties Restore
 	widget = glade_xml_get_widget (gxml, "button16");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_restore_keyboard_properties), gxml);
+	if (!keyboard_properties_config_not_supported)
+		g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_restore_keyboard_properties), gxml);
+	else
+		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE);
 
-	/* release GConf client */
-	g_object_unref(gconfClient);
-	
 	// Button OK
 	widget = glade_xml_get_widget (gxml, "button5");
 	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_save_preference), gxml);
@@ -1387,9 +1346,9 @@ void xneur_add_exclude_app(void)
 	xneur_add_application(store_exclude_app);
 }
 
-void xneur_add_autocomplementation_exclude_app(void)
+void xneur_add_autocompletion_exclude_app(void)
 {
-	xneur_add_application(store_autocomplementation_exclude_app);
+	xneur_add_application(store_autocompletion_exclude_app);
 }
 
 void xneur_add_auto_app(void)
@@ -1442,13 +1401,6 @@ void xneur_add_abbreviation(void)
 	glade_xml_signal_autoconnect (gxml);
 	GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
 
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
-	
 	gtk_widget_show(window);
 	
 	// Button OK
@@ -1492,13 +1444,6 @@ void xneur_add_user_action(void)
 	
 	glade_xml_signal_autoconnect (gxml);
 	GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
-
-	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-	if (window_icon_pixbuf)
-	{
-		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-		gdk_pixbuf_unref (window_icon_pixbuf);
-	}
 	
 	GtkWidget *widget= glade_xml_get_widget (gxml, "entry2");
 	g_signal_connect ((gpointer) widget, "key-press-event", G_CALLBACK (on_key_press_event), gxml);
@@ -1550,13 +1495,6 @@ void xneur_edit_user_action(GtkWidget *treeview)
 		glade_xml_signal_autoconnect (gxml);
 		GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
 		
-		GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-		if (window_icon_pixbuf)
-		{
-			gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-			gdk_pixbuf_unref (window_icon_pixbuf);
-		}
-	
 		char *key_bind;
 		char *user_action;
 		char *action_name;
@@ -1636,13 +1574,6 @@ void xneur_edit_action(GtkWidget *treeview)
 		glade_xml_signal_autoconnect (gxml);
 		GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
 		
-		GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-		if (window_icon_pixbuf)
-		{
-			gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-			gdk_pixbuf_unref (window_icon_pixbuf);
-		}
-	
 		char *key_bind;
 		char *action;
 		gtk_tree_model_get(GTK_TREE_MODEL(store_hotkey), &iter, 0, &action, 1, &key_bind, -1);
@@ -1710,13 +1641,6 @@ void xneur_edit_sound(GtkWidget *treeview)
 		char *file;
 		gtk_tree_model_get(GTK_TREE_MODEL(store_sound), &iter, 1, &file, -1);
 		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window), file);
-		
-		GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-		if (window_icon_pixbuf)
-		{
-			gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-			gdk_pixbuf_unref (window_icon_pixbuf);
-		}
 	
 		gtk_widget_show(window);
 		
@@ -1758,12 +1682,7 @@ void xneur_edit_dictionary(GtkWidget *treeview)
 		}
 	
 		GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
-		GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-		if (window_icon_pixbuf)
-		{
-			gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-			gdk_pixbuf_unref (window_icon_pixbuf);
-		}
+
 		gtk_widget_show(window);
 
 		GtkWidget *widget = glade_xml_get_widget (gxml, "treeview1");
@@ -1861,9 +1780,9 @@ void xneur_rem_exclude_app(GtkWidget *widget)
 	remove_item(widget, store_exclude_app);
 }
 
-void xneur_rem_autocomplementation_exclude_app(GtkWidget *widget)
+void xneur_rem_autocompletion_exclude_app(GtkWidget *widget)
 {
-	remove_item(widget, store_autocomplementation_exclude_app);
+	remove_item(widget, store_autocompletion_exclude_app);
 }
 
 void xneur_rem_auto_app(GtkWidget *widget)
@@ -1905,11 +1824,11 @@ gboolean save_exclude_app(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *i
 	return FALSE;
 }
 
-gboolean save_autocomplementation_exclude_app(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
+gboolean save_autocompletion_exclude_app(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
 {
 	if (model || path || user_data){};
 
-	save_list(store_autocomplementation_exclude_app, xconfig->autocomplementation_excluded_apps, iter);
+	save_list(store_autocompletion_exclude_app, xconfig->autocompletion_excluded_apps, iter);
 
 	return FALSE;
 }
@@ -2154,7 +2073,7 @@ void xneur_save_preference(GladeXML *gxml)
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store_osd), save_osd, NULL);
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store_popup), save_popup, NULL);
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store_hotkey), save_action, NULL);
-	gtk_tree_model_foreach(GTK_TREE_MODEL(store_autocomplementation_exclude_app), save_autocomplementation_exclude_app, NULL);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(store_autocompletion_exclude_app), save_autocompletion_exclude_app, NULL);
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store_plugin), save_plugin, NULL);
 	
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton7");
@@ -2241,15 +2160,33 @@ void xneur_save_preference(GladeXML *gxml)
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton31");
 	xconfig->correct_capital_letter_after_dot = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton29");
+	xconfig->correct_two_space_with_comma_and_space = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton37");
+	xconfig->correct_two_minus_with_dash = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton38");
+	xconfig->correct_c_with_copyright = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton39");
+	xconfig->correct_tm_with_trademark = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton40");
+	xconfig->correct_r_with_registered = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+	
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton21");
-	xconfig->autocomplementation = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+	xconfig->autocompletion = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton1");
-	xconfig->add_space_after_autocomplementation = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+	xconfig->add_space_after_autocompletion = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
 	// Show popup
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton22");
 	xconfig->show_popup = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "spinbutton6");
+	xconfig->popup_expire_timeout = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound));
 	
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "entry2");
 	if (xconfig->osd_font != NULL)
@@ -2271,11 +2208,27 @@ void xneur_save_preference(GladeXML *gxml)
 	xconfig->troubleshoot_delete = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton25");
 	xconfig->troubleshoot_switch = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));;
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton35");
+	xconfig->troubleshoot_full_screen = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));;
 
 	// Dont send KeyRelease mode
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton26");
 	xconfig->dont_send_key_release = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
+	// Tracking input mode
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton34");
+	xconfig->tracking_input = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	// Tracking input mode
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton28");
+	xconfig->tracking_mouse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	// Save
+	xconfig->save(xconfig);
+	xconfig->reload(xconfig);
+	
+	// GXNEUR Preferences
+
 	// Autostart
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton27");
 
@@ -2312,61 +2265,130 @@ void xneur_save_preference(GladeXML *gxml)
 	}
 	g_free(path_file);
 
+	// Gnome 3 Shell Area
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton36");
+
+	path_file = g_build_filename(getenv("HOME"), GNOME3_EXT_PATH, NULL);
+	if (!g_file_test(path_file, G_FILE_TEST_IS_DIR))
+		g_mkdir_with_parents(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+	g_free(path_file);
 	
-	GConfClient* gconfClient = gconf_client_get_default();
-	g_assert(GCONF_IS_CLIENT(gconfClient));
-	// Delay before start
+	path_file = g_build_filename(getenv("HOME"), GNOME3_EXT_PATH, GNOME3_EXT_JS_FILE, NULL);
+	stream = fopen(path_file, "r");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)))
+	{
+		if (stream != NULL)
+		{
+			fclose(stream);
+			remove(path_file);
+		}
+		// Create .js file
+		stream = fopen(path_file, "w");
+		if (stream != NULL)
+		{
+			fprintf(stream, "const StatusIconDispatcher = imports.ui.statusIconDispatcher;\n\n");
+			fprintf(stream, "function enable() {\n");
+			fprintf(stream, "    StatusIconDispatcher.STANDARD_TRAY_ICON_IMPLEMENTATIONS['gxneur'] = 'gxneur';\n");
+			fprintf(stream, "}\n\n");
+			fprintf(stream, "function disable() {\n");
+			fprintf(stream, "    StatusIconDispatcher.STANDARD_TRAY_ICON_IMPLEMENTATIONS['gxneur'] = '';\n");
+			fprintf(stream, "}\n\n");
+			fprintf(stream, "function init() {\n");
+			fprintf(stream, "   StatusIconDispatcher.STANDARD_TRAY_ICON_IMPLEMENTATIONS['gxneur'] = 'gxneur';\n");
+			fprintf(stream, "}\n\n");
+			fprintf(stream, "function main() {\n");
+			fprintf(stream, "   StatusIconDispatcher.STANDARD_TRAY_ICON_IMPLEMENTATIONS['gxneur'] = 'gxneur';\n");
+			fprintf(stream, "}\n\n");
+			fclose(stream);
+		}
+	}
+	else
+	{
+		if (stream != NULL)
+		{
+			fclose(stream);
+			remove(path_file);
+		}
+	}
+	g_free(path_file);
+
+	path_file = g_build_filename(getenv("HOME"), GNOME3_EXT_PATH, GNOME3_EXT_JSON_FILE, NULL);
+	stream = fopen(path_file, "r");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)))
+	{
+		if (stream != NULL)
+		{
+			fclose(stream);
+			remove(path_file);
+		}
+		// Create .json file
+		stream = fopen(path_file, "w");
+		if (stream != NULL)
+		{
+			fprintf(stream, "{\n");
+			fprintf(stream, "   \"shell-version\": [\"3.2\"],\n");
+			fprintf(stream, "   \"uuid\": \"gxneur@xneur.ru\",\n");
+			fprintf(stream, "   \"name\": \"Gxneur\",\n");
+			fprintf(stream, "   \"description\": \"Move the gxneur icon in the shell area of the Gnome 3.\"\n");
+			fprintf(stream, "}\n");
+			fclose(stream);
+		}
+	}
+	else
+	{
+		if (stream != NULL)
+		{
+			fclose(stream);
+			remove(path_file);
+		}
+	}
+	g_free(path_file);
+
+	path_file = g_build_filename(getenv("HOME"), GNOME3_EXT_PATH, NULL);
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)))
+	{
+		g_rmdir(path_file);
+	}
+	g_free(path_file);
+	
+	// Delay
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "spinbutton5");
-	 
-	if(!gconf_client_set_int(gconfClient, PACKAGE_GCONF_DIR "delay", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound)), NULL)) 
-	{
-	    g_warning("Failed to set %s (%d)\n", PACKAGE_GCONF_DIR "delay", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound)));
-	}
+	if (!gxneur_config_write_int("delay", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound)), FALSE))
+		arg_delay = -1;
 
-	// Text on tray
-	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton28");
-	 
-	if(!gconf_client_set_bool(gconfClient, PACKAGE_GCONF_DIR "text_on_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)), NULL)) 
-	{
-	    g_warning("Failed to set %s (%d)\n", PACKAGE_GCONF_DIR "text_on_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)));
-	}
-	gconf_client_notify(gconfClient, PACKAGE_GCONF_DIR "text_on_tray");
+	// Show on the tray
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "combobox2");
+	int show_in_the_tray = gtk_combo_box_get_active(GTK_COMBO_BOX(widgetPtrToBefound));
+	const char *string_value = "Flag";
+	if (show_in_the_tray == 1)
+		string_value = "Text";
+	else if (show_in_the_tray == 2)
+		string_value = "Icon";
+	if (!gxneur_config_write_str("show_in_the_tray", string_value, FALSE))
+		arg_show_in_the_tray = NULL,
+		gxneur_config_write_str("show_in_the_tray", string_value, TRUE);
 
-	// Resize icon on tray
-	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton29");
-	 
-	if(!gconf_client_set_bool(gconfClient, PACKAGE_GCONF_DIR "resize_tray_icon", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)), NULL)) 
-	{
-	    g_warning("Failed to set %s (%d)\n", PACKAGE_GCONF_DIR "resize_tray_icon", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)));
-	}
-	gconf_client_notify(gconfClient, PACKAGE_GCONF_DIR "resize_tray_icon");
-
-	// Path to pixmap dir
-	widgetPtrToBefound = glade_xml_get_widget (gxml, "entry1");
-	
-	if(!gconf_client_set_string(gconfClient, PACKAGE_GCONF_DIR "pixmap_dir", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)), NULL)) 
-	{
-	    g_warning("Failed to set %s (%s)\n", PACKAGE_GCONF_DIR "pixmap_dir", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
-	}
-	gconf_client_notify(gconfClient, PACKAGE_GCONF_DIR "pixmap_dir");
+	// Rendering engine
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "combobox3");
+	int rendering_engine = gtk_combo_box_get_active(GTK_COMBO_BOX(widgetPtrToBefound));
+	string_value = "Built-in";
+	if (rendering_engine == 1)
+		string_value = "StatusIcon";
+	else if (rendering_engine == 2)
+		string_value = "AppIndicator";
+	if (!gxneur_config_write_str("rendering_engine", string_value, FALSE))
+		arg_rendering_engine = NULL,
+		gxneur_config_write_str("rendering_engine", string_value, TRUE);
 
 	// Keyboard properties
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "entry5");
-	
-	if(!gconf_client_set_string(gconfClient, PACKAGE_GCONF_DIR "keyboard_properties", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)), NULL)) 
-	{
-	    g_warning("Failed to set %s (%s)\n", PACKAGE_GCONF_DIR "keyboard_properties", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
-	}
+	if (!gxneur_config_write_str("keyboard_properties", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)), FALSE))
+		arg_keyboard_properties = NULL;
 	add_pixmap_directory(gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
-	
-    /* release GConf client */
-    g_object_unref(gconfClient);
-	
+
+
 	GtkWidget *window = glade_xml_get_widget (gxml, "window2");
 	gtk_widget_destroy(window);
-
-	xconfig->save(xconfig);
-	xconfig->reload(xconfig);
 }
 
 void xneur_dontsave_preference(GladeXML *gxml)
