@@ -40,6 +40,7 @@ char* xneur_modifiers_to_string(unsigned int modifier);
 void xneur_start(void);
 void xneur_preference(void);
 void xneur_kb_preference(void);
+void xneur_get_logfile(void);
 void xneur_about(void);
 
 void xneur_add_exclude_app(void);
@@ -64,8 +65,8 @@ void xneur_add_user_action(void);
 void xneur_rem_user_action(GtkWidget *widget);
 void xneur_edit_user_action(GtkWidget *treeview);
 
-void xneur_add_autocomplementation_exclude_app(void);
-void xneur_rem_autocomplementation_exclude_app(GtkWidget *widget);
+void xneur_add_autocompletion_exclude_app(void);
+void xneur_rem_autocompletion_exclude_app(GtkWidget *widget);
 
 void xneur_edit_action(GtkWidget *treeview);
 void xneur_clear_action(GtkWidget *treeview);
