@@ -20,21 +20,35 @@
 #ifndef _TRAYICON_H_
 #define _TRAYICON_H_
 
+#include "tray_widget.h"
+
 #define MAX_LAYOUTS 4
 
-#include "tray_widget.h"
+#ifdef HAVE_APP_INDICATOR
+#include <libappindicator/app-indicator.h>
+#endif
 
 struct _tray_icon
 {
-	GtkTrayIcon *tray_icon;
+#ifdef HAVE_APP_INDICATOR
+	// App Indicator part
+	AppIndicator *app_indicator;
+#endif
 	
-	GdkPixbuf  *images[MAX_LAYOUTS];
-		
+	// Tray part
+	GtkTrayIcon *tray_icon;
 	GtkTooltips *tooltip;
 	GtkWidget *image;
-	GtkWidget *tray_menu;
 	GtkWidget *evbox;
+	
+	// Status Icon part
+	GtkStatusIcon *status_icon;
 
+	// All
+	gchar  *images[MAX_LAYOUTS];
+	GtkMenu *menu;
+	GtkWidget *status;
+	
 	gint height;
 };
 
